package com.shuto.plugin.tpns;

import android.content.Context;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;

import com.tencent.android.tpush.XGIOperateCallback;
import com.tencent.android.tpush.XGLocalMessage;
import com.tencent.android.tpush.XGPushConfig;
import com.tencent.android.tpush.XGPushManager;

import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaWebView;
import org.apache.cordova.CordovaInterface;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

/**
 * This class echoes a string called from JavaScript.
 */
public class TpnsPlugin extends CordovaPlugin {

    private static final String TAG = TpnsPlugin.class.getSimpleName();

    private Context mContext;
    @Override
    public void initialize(CordovaInterface cordova, CordovaWebView webView) {
        super.initialize(cordova, webView);
        mContext = cordova.getActivity().getApplicationContext();
        CordovaEventKit.init(this);
        Log.d(TAG,"initialize plugin");
    }
    @Override
    public boolean execute(String action, JSONArray args, CallbackContext callbackContext) throws JSONException {
        cordova.getThreadPool().execute(new Runnable() {
            @Override
            public void run() {
                try {
                    Method method = TpnsPlugin.class.getDeclaredMethod(action, JSONArray.class, CallbackContext.class);
                    method.invoke(TpnsPlugin.this, args, callbackContext);
                } catch (Exception e) {
                    Log.e(TAG, e.toString());
                    callbackContext.error(e.getLocalizedMessage());
                }
            }
        });
        return true;
    }

    private void enableDebug(JSONArray data, CallbackContext callbackContext) {
        boolean mode;
        try {
            mode = data.getBoolean(0);
            XGPushConfig.enableDebug(mContext,mode);
            callbackContext.success();
        } catch (JSONException e) {
            e.printStackTrace();
        }
    }

  /**
   * 关闭联合保活能力（1.1.6.1+）
   * @param data
   * @param callbackContext
   */
  private void enablePullUpOtherApp(JSONArray data, CallbackContext callbackContext) {
    boolean mode;
    try {
      mode = data.getBoolean(0);
      XGPushConfig.enablePullUpOtherApp(mContext,mode);
      callbackContext.success();
    } catch (JSONException e) {
      e.printStackTrace();
    }
  }

  /**
   * 获取设备 Token
   * @param data
   * @param callbackContext
   */
  private void getToken(JSONArray data, CallbackContext callbackContext) {
    callbackContext.success(XGPushConfig.getToken(mContext));
  }
  /**
   * 获取第三方厂商 Token
   * @param data
   * @param callbackContext
   */
  private void getOtherPushToken(JSONArray data, CallbackContext callbackContext) {
    callbackContext.success(XGPushConfig.getOtherPushToken(mContext));
  }

    private void registerPush(JSONArray data, CallbackContext callbackContext) {
      try{
        JSONObject params = data.optJSONObject(0);
        if(params !=null){
          XGPushConfig.enableOtherPush(mContext, true);
          Iterator<String> keys = params.keys();
          while (keys.hasNext()) {
            String type = keys.next();
            JSONObject value = params.getJSONObject(type);
            switch (type){
              case "MZ":
                XGPushConfig.setMzPushAppId(mContext, value.getString("APP_ID"));
                XGPushConfig.setMzPushAppKey(mContext, value.getString("APP_KEY"));
                break;
              case "OPPO":
                XGPushConfig.setOppoPushAppId(mContext, value.getString("appKey"));
                XGPushConfig.setOppoPushAppKey(mContext, value.getString("appSecret"));
                break;
              case "MI":
                XGPushConfig.setMiPushAppId(mContext, value.getString("APPID"));
                XGPushConfig.setMiPushAppKey(mContext, value.getString("APPKEY"));
                break;
              case "HW":
                break;
              case "RY":
                break;
            }
          }

        }
      } catch (JSONException e){
        Log.e(TAG,e.getLocalizedMessage());
      }

      XGPushManager.registerPush(mContext, new XGIOperateCallback() {
            @Override
            public void onSuccess(Object data, int flag) {
                Log.d(TAG, "注册成功，设备token为：" + data);
                callbackContext.success((String)data);
            }
            @Override
            public void onFail(Object data, int errCode, String msg) {
                Log.d(TAG, "注册失败，错误码：" + errCode + ",错误信息：" + msg);
                callbackContext.error(msg(errCode,msg));
            }
        });
    }

    private void unregisterPush(JSONArray data, CallbackContext callbackContext) {
        XGPushManager.unregisterPush(mContext, new XGIOperateCallback() {
            @Override
            public void onSuccess(Object data, int i) {
                Log.d(TAG, "反注册成功");
                callbackContext.success();
            }
            @Override
            public void onFail(Object data, int errCode, String msg) {
                Log.d(TAG, "反注册失败，错误码：" + errCode + ",错误信息：" + msg);

                callbackContext.error(msg(errCode,msg));
            }
        });

    }


    private void createNotificationChannel(JSONArray data, CallbackContext callbackContext){
        JSONObject param;
        try {
            param = data.getJSONObject(0);
            XGPushManager.createNotificationChannel(mContext,
                    param.getString("channelId"), param.getString("channelName"),
                    param.optBoolean("enableVibration"), param.optBoolean("enableLights"),
                    param.optBoolean("enableSound"), Uri.parse(param.optString("soundUri")));
            callbackContext.success();
        } catch (JSONException e) {
            e.printStackTrace();
            callbackContext.error(e.getLocalizedMessage());
        }
    }

    private void addLocalNotification(JSONArray data, CallbackContext callbackContext){
        try{
            JSONObject param = data.getJSONObject(0);
            //新建本地通知
            XGLocalMessage local_msg = new XGLocalMessage();
            //设置本地消息类型，1:通知，2:消息
            local_msg.setType(param.optInt("type",1));
            // 设置消息标题
            local_msg.setTitle(param.getString("title"));
            //设置消息内容
            local_msg.setContent(param.getString("content"));
            //设置消息日期，格式为：20140502
            local_msg.setDate(param.optString("date",new SimpleDateFormat("yyyyMMdd ").format(new Date())));
            //设置消息触发的小时(24小时制)，例如：22代表晚上10点
            local_msg.setHour(param.optString("hour",new SimpleDateFormat("HH").format(new Date())));
            //获取消息触发的分钟，例如：05代表05分
            local_msg.setMin(param.optString("min",new SimpleDateFormat("mm").format(new Date())));
            //设置消息样式，默认为0或不设置
            local_msg.setBuilderId(param.optInt("builderId",0));
            //设置动作类型：1打开activity或App本身，2打开浏览器，3打开Intent ，4通过包名打开应用
            int actionType = param.getInt("actionType");
            local_msg.setAction_type(actionType);
            switch (actionType){
                case 1:
                    //设置拉起应用页面
                    local_msg.setActivity( param.getString("activity"));
                    break;
                case 2:
                    // 设置URL
                    local_msg.setUrl(param.getString("url"));
                    break;
                case 3:
                    // 设置Intent
                    local_msg.setIntent(param.getString("intent"));
                    break;
            }

            // 是否覆盖原先build_id的保存设置。1覆盖，0不覆盖
            local_msg.setStyle_id(param.optInt("styleId",1));
            // 设置音频资源
            local_msg.setRing_raw(param.optString("ringRaw"));

            JSONObject body = param.optJSONObject("data");
            // 设置key,value
            HashMap<String, Object> map = new HashMap<String, Object>();
            if(data!=null){
                Iterator<String> keys = body.keys();
                while (keys.hasNext()){
                    String key = keys.next();
                    map.put(key,body.opt(key));
                }
            }
            local_msg.setCustomContent(map);
            //添加通知到本地
            XGPushManager.addLocalNotification(mContext,local_msg);
            callbackContext.success();
        } catch (JSONException e){
            e.printStackTrace();
            callbackContext.error(e.getLocalizedMessage());
        }
    }

    private void clearLocalNotifications(JSONArray data, CallbackContext callbackContext){
        XGPushManager.clearLocalNotifications(mContext);
        callbackContext.success();
    }


    /**
     * 对一个或多个账号类型的账号进行解绑。（SDK 1.2.3.0+）
     * eg:[1001,1002]
     * @param data
     */
    private void delAccounts(JSONArray data, CallbackContext callbackContext) {
        Log.d(TAG, "delAccounts | params: " + data);
        try {
            Set<Integer> accountInfos = new HashSet<>();
            JSONArray accInfoArr = data.getJSONArray(0);
            for(int i=0; i < accInfoArr.length(); i++) {
                int accountType = accInfoArr.optInt(i,0);
                accountInfos.add(accountType);
            }
            XGPushManager.delAccounts(mContext, accountInfos,new XGIOperateCallback() {
                @Override
                public void onSuccess(Object data, int i) {
                    Log.d(TAG, "对一个或多个账号类型的账号进行解绑成功");
                    callbackContext.success();
                }
                @Override
                public void onFail(Object data, int errCode, String msg) {
                    Log.d(TAG, "对一个或多个账号类型的账号进行解绑失败，错误码：" + errCode + ",错误信息：" + msg);
                    callbackContext.error(msg(errCode,msg));
                }
            });
        } catch (JSONException e) {
            Log.d(TAG, "setAccount exception : " + e.getMessage());
            callbackContext.error(e.getLocalizedMessage());
        }
    }

    /**
     * 对的所有已绑定账号进行解绑。
     * @param data
     */
    private void clearAccounts(JSONArray data, CallbackContext callbackContext) {
        Log.d(TAG, "clearAccounts | params: " + data);
        XGPushManager.clearAccounts(mContext,new XGIOperateCallback() {
            @Override
            public void onSuccess(Object data, int i) {
                Log.d(TAG, "对的所有已绑定账号进行解绑成功");
                callbackContext.success();
            }
            @Override
            public void onFail(Object data, int errCode, String msg) {
                Log.d(TAG, "对的所有已绑定账号进行解绑失败，错误码：" + errCode + ",错误信息：" + msg);
                callbackContext.error(msg(errCode,msg));
            }
        });
    }

    /**
     * 添加或更新账号。若原来没有该类型账号，则添加；若原来有，则覆盖。可以同时添加多个账号，一个账号对应一个账号类型。
     * eg:[{"account":elva, "accountType":1001}, {"account":jay, "accountType":1002}]
     * @param data
     */
    private void upsertAccounts(JSONArray data, CallbackContext callbackContext) {
        Log.d(TAG, "upsertAccounts | params: " + data);
        try {
            List<XGPushManager.AccountInfo> accountInfos = new ArrayList<>();
            JSONArray accInfoArr = data.getJSONArray(0);
            for(int i=0; i < accInfoArr.length(); i++) {
                JSONObject accInfoJo = accInfoArr.optJSONObject(i);
                if(accInfoJo == null) {
                    continue;
                }
                String account = accInfoJo.optString("account","");
                int accountType = accInfoJo.optInt("accountType", 0);
                XGPushManager.AccountInfo info = new XGPushManager.AccountInfo(accountType, account);
                accountInfos.add(info);
            }
            XGPushManager.upsertAccounts(mContext, accountInfos,new XGIOperateCallback() {
                @Override
                public void onSuccess(Object data, int i) {
                    Log.d(TAG, "添加或更新账号成功");
                    callbackContext.success();
                }
                @Override
                public void onFail(Object data, int errCode, String msg) {
                    Log.d(TAG, "添加或更新账号失败，错误码：" + errCode + ",错误信息：" + msg);
                    callbackContext.error(msg(errCode,msg));
                }
            });
        } catch (JSONException e) {
            Log.d(TAG, "setAccount exception : " + e.getMessage());
            callbackContext.error(e.getLocalizedMessage());
        }
    }

    /**
     * 覆盖多个标签 一次设置多个标签，会覆盖这个设备之前设置的标签。
     */
    private void clearAndAppendTags(JSONArray data, CallbackContext callbackContext) {
        Log.d(TAG, "clearAndAppendTags | params: " + data);
        try {
            Set<String> tags = new HashSet<>();
          String operateName = data.getString(0);
            JSONArray accInfoArr = data.getJSONArray(1);
            for(int i=0; i < accInfoArr.length(); i++) {
                String tag = accInfoArr.optString(i);
                tags.add(tag);
            }
            XGPushManager.clearAndAppendTags(mContext,operateName, tags,new XGIOperateCallback() {
                @Override
                public void onSuccess(Object data, int i) {
                    Log.d(TAG, "覆盖多个标签成功");
                    callbackContext.success();
                }
                @Override
                public void onFail(Object data, int errCode, String msg) {
                    Log.d(TAG, "覆盖多个标签失败，错误码：" + errCode + ",错误信息：" + msg);
                    callbackContext.error(msg(errCode,msg));
                }
            });
        } catch (JSONException e) {
            Log.d(TAG, "setAccount exception : " + e.getMessage());
            callbackContext.error(e.getLocalizedMessage());
        }
    }

    /**
     * 新增多个标签
     * 如果新覆盖的标签都带有 : 号，例如 test:2, level:2，则会删除这个设备已绑定的所有 test:* 和 level:* 标签，再新增 test:2 和 level:2。
     * 如果新增的标签有部分不带 : 号，例如 test:2 level，则会删除这个设备的全部历史标签，再新增 test:2 和 level 标签。
     */
    private void appendTags(JSONArray data, CallbackContext callbackContext) {
        Log.d(TAG, "appendTags | params: " + data);
        try {
            Set<String> tags = new HashSet<>();
          String operateName = data.getString(0);
            JSONArray accInfoArr = data.getJSONArray(1);
            for(int i=0; i < accInfoArr.length(); i++) {
                String tag = accInfoArr.optString(i);
                tags.add(tag);
            }
            XGPushManager.appendTags(mContext,operateName, tags,new XGIOperateCallback() {
                @Override
                public void onSuccess(Object data, int i) {
                    Log.d(TAG, "新增多个标签成功");
                    callbackContext.success();
                }
                @Override
                public void onFail(Object data, int errCode, String msg) {
                    Log.d(TAG, "新增多个标签失败，错误码：" + errCode + ",错误信息：" + msg);
                    callbackContext.error(msg(errCode,msg));
                }
            });
        } catch (JSONException e) {
            Log.d(TAG, "setAccount exception : " + e.getMessage());
            callbackContext.error(e.getLocalizedMessage());
        }
    }
    /**
     * 一次删除多个标签。
     */
    private void delTags(JSONArray data, CallbackContext callbackContext) {
        Log.d(TAG, "delTags | params: " + data);
        try {
            Set<String> tags = new HashSet<>();
          String operateName = data.getString(0);
            JSONArray accInfoArr = data.getJSONArray(1);
            for(int i=0; i < accInfoArr.length(); i++) {
                String tag = accInfoArr.optString(i);
                tags.add(tag);
            }
            XGPushManager.delTags(mContext,operateName, tags,new XGIOperateCallback() {
                @Override
                public void onSuccess(Object data, int i) {
                    Log.d(TAG, "一次删除多个标签成功");
                    callbackContext.success();
                }
                @Override
                public void onFail(Object data, int errCode, String msg) {
                    Log.d(TAG, "一次删除多个标签失败，错误码：" + errCode + ",错误信息：" + msg);
                    callbackContext.error(msg(errCode,msg));
                }
            });
        } catch (JSONException e) {
            Log.d(TAG, "setAccount exception : " + e.getMessage());
            callbackContext.error(e.getLocalizedMessage());
        }
    }

    /**
     * 清除这个设备的所有标签。
     */
    private void clearTags(JSONArray data, CallbackContext callbackContext) {
        Log.d(TAG, "clearTags | params: " + data);
        try {
            String operateName = data.getString(0);
            XGPushManager.clearTags(mContext,operateName,new XGIOperateCallback() {
                @Override
                public void onSuccess(Object data, int i) {
                    Log.d(TAG, "清除这个设备的所有标签成功");
                    callbackContext.success();
                }
                @Override
                public void onFail(Object data, int errCode, String msg) {
                    Log.d(TAG, "清除这个设备的所有标签失败，错误码：" + errCode + ",错误信息：" + msg);
                    callbackContext.error(msg(errCode,msg));
                }
            });
        } catch (JSONException e) {
            Log.d(TAG, "setAccount exception : " + e.getMessage());
            callbackContext.error(e.getLocalizedMessage());
        }
    }

    /**
     * 获取这个设备的标签
     */
    private void queryTags(JSONArray data, CallbackContext callbackContext) {
        Log.d(TAG, "queryTags | params: " + data);
        try {
            String operateName = data.getString(0);
            int start = data.optInt(1,0);
            int size = data.optInt(2,100);
            Events.addListener("queryTags." + operateName, new Listener() {
              @Override
              public void on(Bundle extra) {
                Log.d(TAG,"接收到查询标签结果");
                callbackContext.success(msg(extra.getInt("code"),extra.getString("data")));
                Events.removeListener("queryTags." + operateName,this);
              }
            });
            XGPushManager.queryTags(mContext,operateName,start,size,new XGIOperateCallback() {
                @Override
                public void onSuccess(Object data, int i) {
                    Log.d(TAG, "清除这个设备的所有标签成功");
                }
                @Override
                public void onFail(Object data, int errCode, String msg) {
                    Log.d(TAG, "清除这个设备的所有标签失败，错误码：" + errCode + ",错误信息：" + msg);
                    callbackContext.error(msg(errCode,msg));
                }
            });
        } catch (JSONException e) {
            Log.d(TAG, "setAccount exception : " + e.getMessage());
            callbackContext.error(e.getLocalizedMessage());
        }
    }

  /**
   * 添加属性（带回调）：有则覆盖，无则添加。
   */
  private void upsertAttributes(JSONArray data, CallbackContext callbackContext) {
    Log.d(TAG, "upsertAttributes | params: " + data);
    try {
      String operateName = data.getString(0);
      JSONObject body = data.optJSONObject(1);
      HashMap<String, String> map = new HashMap<>();
      if(data!=null){
        Iterator<String> keys = body.keys();
        while (keys.hasNext()){
          String key = keys.next();
          map.put(key,body.optString(key));
        }
      }
      XGPushManager.upsertAttributes(mContext,operateName,map,new XGIOperateCallback() {
        @Override
        public void onSuccess(Object data, int i) {
          Log.d(TAG, "添加属性成功");
        }
        @Override
        public void onFail(Object data, int errCode, String msg) {
          Log.d(TAG, "添加属性失败，错误码：" + errCode + ",错误信息：" + msg);
          callbackContext.error(msg(errCode,msg));
        }
      });
    } catch (JSONException e) {
      Log.d(TAG, "setAccount exception : " + e.getMessage());
      callbackContext.error(e.getLocalizedMessage());
    }
  }

  /**
   * 更新用户属性
   */
  private void clearAndAppendAttributes(JSONArray data, CallbackContext callbackContext) {
    Log.d(TAG, "clearAndAppendAttributes | params: " + data);
    try {
      String operateName = data.getString(0);
      JSONObject body = data.optJSONObject(1);
      HashMap<String, String> map = new HashMap<>();
      if(data!=null){
        Iterator<String> keys = body.keys();
        while (keys.hasNext()){
          String key = keys.next();
          map.put(key,body.optString(key));
        }
      }
      XGPushManager.clearAndAppendAttributes(mContext,operateName,map,new XGIOperateCallback() {
        @Override
        public void onSuccess(Object data, int i) {
          Log.d(TAG, "更新用户属性成功");
        }
        @Override
        public void onFail(Object data, int errCode, String msg) {
          Log.d(TAG, "更新用户属性失败，错误码：" + errCode + ",错误信息：" + msg);
          callbackContext.error(msg(errCode,msg));
        }
      });
    } catch (JSONException e) {
      Log.d(TAG, "setAccount exception : " + e.getMessage());
      callbackContext.error(e.getLocalizedMessage());
    }
  }

  /**
   * 删除指定的属性。
   *
   */
  private void delAttributes(JSONArray data, CallbackContext callbackContext) {
    Log.d(TAG, "delAttributes | params: " + data);
    try {
      String operateName = data.getString(0);
      Set<String> attrs = new HashSet<>();
      JSONArray accInfoArr = data.getJSONArray(1);
      for(int i=0; i < accInfoArr.length(); i++) {
        String tag = accInfoArr.optString(i);
        attrs.add(tag);
      }
      XGPushManager.delAttributes(mContext,operateName, attrs,new XGIOperateCallback() {
        @Override
        public void onSuccess(Object data, int i) {
          Log.d(TAG, "删除指定的属性成功");
          callbackContext.success();
        }
        @Override
        public void onFail(Object data, int errCode, String msg) {
          Log.d(TAG, "删除指定的属性失败，错误码：" + errCode + ",错误信息：" + msg);
          callbackContext.error(msg(errCode,msg));
        }
      });
    } catch (JSONException e) {
      Log.d(TAG, "setAccount exception : " + e.getMessage());
      callbackContext.error(e.getLocalizedMessage());
    }
  }

  /**
   * 删除已设置的所有属性。
   */
  private void clearAttributes(JSONArray data, CallbackContext callbackContext) {
    Log.d(TAG, "clearAttributes | params: " + data);
    try {
      String operateName = data.getString(0);
      XGPushManager.clearAttributes(mContext,operateName,new XGIOperateCallback() {
        @Override
        public void onSuccess(Object data, int i) {
          Log.d(TAG, "删除已设置的所有属性成功");
          callbackContext.success();
        }
        @Override
        public void onFail(Object data, int errCode, String msg) {
          Log.d(TAG, "删除已设置的所有属性失败，错误码：" + errCode + ",错误信息：" + msg);
          callbackContext.error(msg(errCode,msg));
        }
      });
    } catch (JSONException e) {
      Log.d(TAG, "setAccount exception : " + e.getMessage());
      callbackContext.error(e.getLocalizedMessage());
    }
  }

  /**
     *
     * @param data
     * @param callbackContext
     */

    private void cancelAllNotifaction(JSONArray data, CallbackContext callbackContext){
        XGPushManager.cancelAllNotifaction(mContext);
        callbackContext.success();
    }








    private JSONObject msg(int code, String msg){
        JSONObject obj = new JSONObject();
        try {
            obj.put("code",code);
            obj.put("message",msg);
        }catch (JSONException e){
            Log.e(TAG,e.getMessage());
        }finally {
            return obj;
        }
    }

}
